const lo = require('lodash')
const hlp = require('../../../helpers.js')
const hlp_t = require('../../../helpers_time.js')

const formValues = ['Enable', 'Frequency',
    'EnableDurationLimit','EnableSuspend','EnableDeactivate','RunMissedTasks','EnableRandomizationPeriod','RandomizationPeriod']

function defaultTaskSchedule() {
    return {
        ActivateTime: {type: "long", value: "0" },
        DaysOfWeek: 2,
        DeactivateTime: {type: "long", value: "0" },
        DurationLimit: 60,
        Enable: false,
        EnableDeactivate: false,
        EnableDurationLimit: false,
        EnableRandomizationPeriod: false,
        EnableSuspend: false,
        Frequency: 0,
        RandomizationPeriod: 60,
        RunMissedTasks: false,
        StartTime: 0,
        SuspendFrom: 0,
        SuspendTill: 0,
        UnitCount: 0,
        __VersionInfo: [1, 0]
    }
}

function PolicyToLocalSchedule(item) {
    Object.keys(item).forEach(property => {
        item[property] = item[property].KLPRSS_Val;
    })
    return item;
}

////////////////////////////////////////////////////
function toView(schSett, schPolicy) {
    if(schSett===undefined )
        schSett = defaultTaskSchedule();

    const bMandatory = lo.get(schPolicy, 'KLPRSS_Mnd', false);
    const bLocked    = (lo.get(schPolicy, 'KLPRSS_ValLck', false)===true || bMandatory);
    const ScheduleData = bMandatory ? PolicyToLocalSchedule(schPolicy.KLPRSS_Val[0].Schedule.KLPRSS_Val) : schSett;
    const uCount = (ScheduleData.UnitCount===0 ) ? 1 : ScheduleData.UnitCount
    let data = {
        LocalTask:true,
        nHour: uCount,
        nDay: uCount,
        nWeek: uCount,
        ActivateDate: hlp_t.GetDateUtc(ScheduleData.ActivateTime.value),
        DeactivateTime: hlp_t.GetDateUtc(ScheduleData.DeactivateTime.value),
        StartTime: hlp_t.SecondsToTimeStr(ScheduleData.StartTime),
        SuspendFrom: hlp_t.SecondsToTimeStr(ScheduleData.SuspendFrom),
        SuspendTill: hlp_t.SecondsToTimeStr(ScheduleData.SuspendTill),
        DurationLimit: hlp_t.MinutesToTimeStr(ScheduleData.DurationLimit),
        allScheduleData: { KLPRSS_Mnd: bMandatory, KLPRSS_ValLck: bLocked }
    }

    for (let key of formValues)
        data[key] = ScheduleData[key]

    hlp_t.DaysOfWeektoView(data, ScheduleData.DaysOfWeek)
    return data
}

function toRaw (changedData, schSett) {
    if(schSett=== undefined )
        return

    for (let key of formValues) {
        schSett[key] = changedData[key]
    }

    schSett.DaysOfWeek = hlp_t.DaysOfWeektoRow(changedData)
    schSett.ActivateTime.value = hlp_t.SetDateUtc(changedData.ActivateDate)
    schSett.StartTime= hlp_t.TimeStrToSeconds(changedData.StartTime)

    if(changedData.Frequency=== hlp_t.TaskScheduleFrequencyEnum.Hours )
        schSett.UnitCount = changedData.nHour
    else if(changedData.Frequency=== hlp_t.TaskScheduleFrequencyEnum.Days )
        schSett.UnitCount = changedData.nDay
    else if(changedData.Frequency=== hlp_t.TaskScheduleFrequencyEnum.Weeks )
        schSett.UnitCount = changedData.nWeek

    if(changedData.EnableDeactivate)
        schSett.DeactivateTime.value = hlp_t.SetDateUtc(changedData.DeactivateTime);
    if(changedData.EnableSuspend){
        schSett.SuspendFrom = hlp_t.TimeStrToSeconds(changedData.SuspendFrom)
        schSett.SuspendTill = hlp_t.TimeStrToSeconds(changedData.SuspendTill)
    }
    if(changedData.EnableDurationLimit)
        schSett.DurationLimit = hlp_t.TimeStrToMinutes(changedData.DurationLimit)
}

module.exports = {
    toView,
    toRaw
};
