const lo = require('lodash')
const fs = require('fs')
const path = require('path')
const hlp = require('../../helpers')
const hlpPolicy = require('../../helpers_policy')
const consts = require('../../consts')
const { plugin: { pluginScope } } = require('@kl/constants')

const policyConverters = require('./sections/index')
const policySections = consts.policySsSections

function Policy () {
    let changesList = []
    let policiesCache = {}
    let curPolicyId = -1

    function AddToChangesList (arrSections) {
        arrSections = Array.isArray(arrSections) ? arrSections : [arrSections]
        for (let policySection of arrSections) {
            if (policySection != '' && !changesList.includes(policySection)) {
                changesList.push(policySection)
            }
        }
    }

    function UpdatePoliciesCacheAsDefault (arrSections) {
        arrSections = Array.isArray(arrSections) ? arrSections : [arrSections]
        for (let policySection of arrSections) {
            if (policySection != '' && policiesCache[policySection] === undefined ) {
                policiesCache[policySection] = hlpPolicy.DefaultPolicyObject(policySection)
            }
        }
    }

    /*
     * Метод добавляет в cache политик новые секции
     */
    async function UpdatePoliciesCache (api, id, arrSections) {
        arrSections = Array.isArray(arrSections) ? arrSections : [arrSections]

        if (id === -1) {
            UpdatePoliciesCacheAsDefault(arrSections)
            return
        }

        let { result: identifier } = await api.Policy.GetPolicyContents({ nPolicy: id })

        for (let i = 0; i < arrSections.length; i++) {
            const policySection = arrSections[i]
            if( policiesCache[policySection] !== undefined )
                continue

            const SsPolicySection = policySections[policySection]
            if(SsPolicySection === undefined)
                continue

            let sectionData = await api.SsContents.Ss_Read({
                wstrID: identifier,
                wstrProduct: consts.wstrProduct,
                wstrVersion: consts.wstrVersion,
                wstrSection: SsPolicySection,
                skipDeserialization: true})

            let error = lo.get(sectionData, 'PxgError')
            if (error) {
                policiesCache[policySection] = hlpPolicy.DefaultPolicyObject(policySection)
            }
            else {
                let rowData
                if(sectionData=== undefined)
                {
                    rowData = hlpPolicy.DefaultPolicyObject(policySection)
                }
                else
                {
                    let deserealized = hlp.deserialize(lo.get(sectionData, 'result'));
                    rowData = deserealized;
                }

                policiesCache[policySection] = rowData
            }
        }
        api.SsContents.Ss_Release({ wstrID: identifier })

        return
    }

    async function SaveDefaultPolicySections (api, id) {
        let res = {}
        if( id === -1 )
            return res

        let { result: identifier } = await api.Policy.GetPolicyContents({ nPolicy: id })

        for (let keySection in policySections) {
            if( changesList[keySection] !== undefined )
                continue;

            const SsPolicySection = policySections[keySection]
            let DataToSave = hlpPolicy.DefaultPolicyObject (keySection)
            if( DataToSave.length === 0 )
                continue;

            let result = hlp.serialize(DataToSave)

            await api.SsContents.Ss_Replace({
                wstrID: identifier,
                wstrProduct: consts.wstrProduct,
                wstrVersion: consts.wstrVersion,
                wstrSection: SsPolicySection,
                pNewData: result,
                skipSerialization: true
            })
            await api.SsContents.Ss_Apply({ wstrID: identifier })
        }
        api.SsContents.Ss_Release({ wstrID: identifier })
        return res
    }
    /*
     * Called via api.js in getFormModel
     */
    async function getPolicyData (api, id, section) {
        if(curPolicyId != id) {
            curPolicyId = id
            policiesCache = {}
            changesList = []
        }

        if (section === undefined || policyConverters[section] === undefined)
            return {}

        if (policyConverters[section].sections !== undefined) {
            await UpdatePoliciesCache(api, id, policyConverters[section].sections);
        }

        if (policyConverters[section].toViewEx !== undefined) {
            return await policyConverters[section].toViewEx(api, policiesCache)
        }

        if (policyConverters[section].toView !== undefined) {
            return policyConverters[section].toView(policiesCache)
        }

        return {}
    }
    /*
     * Called via api.js in setFormModel
     */
    async function updatePolicyData (api, section, data) {
        if (policyConverters[section] === undefined)
            return
        if (policyConverters[section].sections !== undefined) {
            AddToChangesList(policyConverters[section].sections)
        }

        if(policyConverters[section].toRawEx!==undefined)
            await policyConverters[section].toRawEx(api, data, policiesCache)
        else
            policyConverters[section].toRaw(data, policiesCache)
    }
    /*
     * Called via api.js in beforeApplyChanges
     */
    function commitPolicyData (api, scope) {
    }
    /*
     * Called via api.js in applyChanges
     */
    async function savePolicyData (api, id, scope) {
        if (scope === pluginScope.newObject) {
            SaveDefaultPolicySections (api, id)
        }

        if(changesList.length === 0 )
            return

        let { result: identifier } = await api.Policy.GetPolicyContents({ nPolicy: id })

        for (let policySection of changesList) {
            const SsPolicySection = policySections[policySection]
            let DataToSave = {}
            DataToSave = policiesCache[policySection]
            let result = hlp.serialize(DataToSave)

			console.log('soyuz_windows:> Save policy section ' + policySection)
			
            await api.SsContents.Ss_Replace({
                wstrID: identifier,
                wstrProduct: consts.wstrProduct,
                wstrVersion: consts.wstrVersion,
                wstrSection: SsPolicySection,
                pNewData: result,
                skipSerialization: true
            })
            await api.SsContents.Ss_Apply({ wstrID: identifier })
			console.log('soyuz_windows:> Saved policy section ' + policySection)
        }

        api.SsContents.Ss_Release({ wstrID: identifier })

        changesList = []
		policiesCache = {}
    }

    return {
        getPolicyData,
        updatePolicyData,
        commitPolicyData,
        savePolicyData
    }
}

module.exports = Policy
