//const hlp = require('./helpers.js')
const TaskScheduleFrequencyEnum = {Never:0, Hours:1, Days:2, Weeks:3, AtApplicationStart:4, AfterAVBasesUpdate:5, Once:6, AdminKitControlledSchedule:100 }
const TaskScheduleDaysOfWeek = { Mon: 2, Tue: 4, Wed: 8, Thu: 16, Fri: 32,  Sat: 64, Sun: 1 }

// Adds two arrays for the given base (10 or 16), returning the result.
// This turns out to be the only "primitive" operation we need.
function add(x, y, base) {
    var z = [];
    var n = Math.max(x.length, y.length);
    var carry = 0;
    var i = 0;
    while (i < n || carry) {
        var xi = i < x.length ? x[i] : 0;
        var yi = i < y.length ? y[i] : 0;
        var zi = carry + xi + yi;
        z.push(zi % base);
        carry = Math.floor(zi / base);
        i++;
    }
    return z;
}

// Returns a*x, where x is an array of decimal digits and a is an ordinary
// JavaScript number. base is the number base of the array x.
function multiplyByNumber(num, x, base) {
    if (num < 0) return null;
    if (num == 0) return [];

    var result = [];
    var power = x;
    while (true) {
        if (num & 1) {
            result = add(result, power, base);
        }
        num = num >> 1;
        if (num === 0) break;
        power = add(power, power, base);
    }

    return result;
}

function parseToDigitsArray(str, base) {
    var digits = str.split('');
    var ary = [];
    for (var i = digits.length - 1; i >= 0; i--) {
        var n = parseInt(digits[i], base);
        if (isNaN(n)) return null;
        ary.push(n);
    }
    return ary;
}

function convertBase(str, fromBase, toBase) {
    var digits = parseToDigitsArray(str, fromBase);
    if (digits === null) return null;

    var outArray = [];
    var power = [1];
    for (var i = 0; i < digits.length; i++) {
        // invariant: at this point, fromBase^i = power
        if (digits[i]) {
            outArray = add(outArray, multiplyByNumber(digits[i], power, toBase), toBase);
        }
        power = multiplyByNumber(fromBase, power, toBase);
    }

    var out = '';
    for (var i = outArray.length - 1; i >= 0; i--) {
        out += outArray[i].toString(toBase);
    }
    return out;
}

function GetDateUtc(str) {
    let hex = convertBase(str, 10, 16);
    if( hex===undefined || hex.length != 15 ) {
        let today = new Date()
        return Date.UTC(today.getFullYear(), today.getMonth(), today.getDate())
    }

    let Year = parseInt(hex.substr(0, 3), 16);
    let Month = parseInt(hex.substr(3, 2));
    if(Month > 0)
        Month -= 1
    let Day = parseInt(hex.substr(5, 2));

    return Date.UTC(Year, Month, Day)
}

function SetDateUtc(utc) {
    let date = new Date(utc)
    let month = date.getMonth() + 1
    let hex = date.getFullYear().toString(16)
    hex += "0"
    hex += month.toString(16)
    hex += "0"
    hex += date.getDate().toString(16)
    hex += "0000000"
    hex += date.getDay().toString(16)
    hex = hex.toLowerCase();
    return convertBase(hex, 16, 10);
}

function SecondsToTimeStr(seconds, setToday = false) {
    let hh = 0
    let mm = 0
    if(seconds > 0 ){
        hh = Math.floor(seconds/3600);
        mm = Math.floor((seconds - hh*3600)/60);
    }
    else if( setToday){
        var today = new Date();
        hh = today.getUTCHours();
        mm = today.getUTCMinutes();
    }
    return (hh + ":" + mm)
}

function MinutesToTimeStr(minutes) {
    let hh = 0
    let mm = 0
    if(minutes > 0 ){
        hh = Math.floor(minutes/60);
        mm = (minutes - hh*60);
    }
    return hh + ":" + mm;
}

function TimeStrToSeconds(str) {
    let seconds = 0
    let arr = str.split(":").map(val => Number(val));
    if (arr[0] >= 0 && arr[0] <= 23 ) {
        seconds = 3600 * arr[0]
    }
    if (arr[1] >= 0 && arr[1] <= 59 ) {
        seconds += 60 * arr[1]
    }

    return seconds
}

function TimeStrToMinutes(str) {
    let minutes = 0
    let arr = str.split(":").map(val => Number(val));
    if (arr[0] >= 0 && arr[0] <= 23 ) {
        minutes = 60 * arr[0]
    }
    if (arr[1] >= 0 && arr[1] <= 59 ) {
        minutes += arr[1]
    }

    return minutes
}

function DaysOfWeektoView(data, daysOfWeek) {
    data.Monday = (daysOfWeek & TaskScheduleDaysOfWeek.Mon) ? true : false;
    data.Tuesday = (daysOfWeek & TaskScheduleDaysOfWeek.Tue) ? true : false;
    data.Wednesday = (daysOfWeek & TaskScheduleDaysOfWeek.Wed) ? true : false;
    data.Thursday = (daysOfWeek & TaskScheduleDaysOfWeek.Thu) ? true : false;
    data.Friday = (daysOfWeek & TaskScheduleDaysOfWeek.Fri) ? true : false;
    data.Saturday = (daysOfWeek & TaskScheduleDaysOfWeek.Sat) ? true : false;
    data.Sunday = (daysOfWeek & TaskScheduleDaysOfWeek.Sun) ? true : false;
}

function DaysOfWeektoRow(data) {
    let daysOfWeek = 0
    if( data.Monday )
        daysOfWeek |= TaskScheduleDaysOfWeek.Mon;
    if( data.Tuesday )
        daysOfWeek |= TaskScheduleDaysOfWeek.Tue;
    if( data.Wednesday )
        daysOfWeek |= TaskScheduleDaysOfWeek.Wed;
    if( data.Thursday )
        daysOfWeek |= TaskScheduleDaysOfWeek.Thu;
    if( data.Friday )
        daysOfWeek |= TaskScheduleDaysOfWeek.Fri;
    if( data.Saturday )
        daysOfWeek |= TaskScheduleDaysOfWeek.Sat;
    if( data.Sunday )
        daysOfWeek |= TaskScheduleDaysOfWeek.Sun;
    return daysOfWeek
}

module.exports = {
    convertBase,
    TaskScheduleFrequencyEnum,
    GetDateUtc,
    SetDateUtc,
    SecondsToTimeStr,
    MinutesToTimeStr,
    TimeStrToSeconds,
    TimeStrToMinutes,
    DaysOfWeektoView,
    DaysOfWeektoRow
}